@@ -121,12 +121,14 @@ void UserAccount::reset()
 	tog_agents = true;
 	tog_self = false;
 	customAuth = false;
+	storeSaltedHashedPassword = false;
 	req_mutual_auth = false;
 	legacy_ssl_probe = false;
 	security_level = QCA::SL_None;
 	ssl = SSL_Auto;
 	jid = "";
 	pass = "";
+	scramSaltedHashPassword = "";
 	opt_pass = false;
 	port = 5222;
 	opt_host = false;
@@ -231,6 +233,10 @@ void UserAccount::fromOptions(OptionsTree *o, QString base)
 	authid = o->getOption(base + ".custom-auth.authid").toString();
 	realm = o->getOption(base + ".custom-auth.realm").toString();
 
+	// read scram salted password options
+	storeSaltedHashedPassword = o->getOption(base + ".scram.store-salted-password").toBool();
+	scramSaltedHashPassword = o->getOption(base + ".scram.salted-password").toString();
+
 	// read password (we must do this after reading the jid, to decode properly)
 	QString tmp = o->getOption(base + ".password").toString();
 	if(!tmp.isEmpty()) {
@@ -384,6 +390,9 @@ void UserAccount::toOptions(OptionsTree *o, QString base)
 	o->setOption(base + ".custom-auth.authid", authid);
 	o->setOption(base + ".custom-auth.realm", realm);
 
+	o->setOption(base + ".scram.store-salted-password", storeSaltedHashedPassword);
+	o->setOption(base + ".scram.salted-password", scramSaltedHashPassword);
+
 	if(opt_pass) {
 		o->setOption(base + ".password", encodePassword(pass, jid));
 	} else {
@@ -1530,7 +1539,7 @@ bool profileRename(const QString &oldname, const QString &name)
 	// and if all ok we may rename it.
 	foreach(QString path, paths) {
 		QDir d(path);
-		if(!d.exists())
+		if(!d.exists() || !d.exists(oldname))
 			continue;
 
 		if(!d.rename(oldname, name))
