@@ -61,6 +61,7 @@ private slots:
 	QString makeContactName(const QString &name, const Jid &jid) const;
 	void doSwitchJidMode();
 	void actActiveContacts();
+	void actPgpToggled(bool);
 
 private:
 	void initToolBar();
@@ -75,6 +76,7 @@ private:
 	void updateAvatar();
 	void optionsUpdate();
 	void updatePGP();
+	void checkPGPAutostart();
 	void setPGPEnabled(bool enabled);
 	void activated();
 	void setLooks();
@@ -122,6 +124,8 @@ private:
 	int logHeight;
 	int chateditHeight;
 
+	bool autoPGP_;
+
 private:
 	bool tabmode;
 	QPointer <PsiWindowHeader> winHeader_;
