@@ -347,7 +347,7 @@ QString UserListItem::findClient(QString name) const
 	QString res;
 	if(name.startsWith("adium"))
 		res = "adium";
-	else if(name.startsWith("google talk user account"))
+	else if(name.startsWith("google talk user account") || name.contains("android.com"))
 		res = "android";
 	else if(name.startsWith("aqq"))
 		res = "aqq";
@@ -389,6 +389,8 @@ QString UserListItem::findClient(QString name) const
 		res = "coccinella";
 	else if(name.startsWith("dictbot"))
 		res = "dictbot";
+	else if(name.startsWith("Digsby Client"))
+		res = "digsby";
 	else if(name.startsWith("ekg2"))
 		res = "ekg2";
 	else if(name.startsWith("emess"))
